#include "ti86asm.inc"

.org _asm_exec_ram

	nop
	jp ProgStart
	.dw 0
	.dw ShellTitle

ShellTitle:
	.db "Ben Weber's Scrolling Demo",0

ProgStart:
 call _runindicoff
 ld a,0
 ld (Tile),a
 ld (Xoffset),a

ClrMem:												;clear video mem and graph mem
 call _clrLCD
ClrGrf:												;clear graph mem only
 ld hl,_plotSScreen+6								;$ca00
 ld d,h
 ld e,l
 inc de
 ld (hl),0
 ld bc,1024
 ldir												;clear it


 ld hl,TitleSprite
 ld de,Buffer
 ld bc,1024
 ldir

 ld hl,Buffer
 ld de,$Fc00
 ld bc,1024
 ldir
 call _getkey

Keys:

 ld hl,Buffer
 ld de,$Fc00
 ld bc,1024
 ldir

 halt
 halt
 halt
 halt
 call ScrollL
 call _getky
 cp K_EXIT
 jp z,Exit
 ld a,(Xoffset)
 inc a
 ld (Xoffset),a
 ld b,a
 ld a,8
 cp b
 jp z,DrawCol
 jp Keys
DrawCol:
 ld hl,Map1
 call PutCol
 jp Keys

Wait:
 call _getkey
 ret
 ret
 ret
Exit:
 ret

ScrollL:       ; calling this routine will scroll the whole screen to the left by 1 pixel
 ld hl,Buffer + 1023
 ld b,64
SLLoop1:
 or a
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 djnz SLLoop1
 ret

;============================================================
; DrawMap -- draws an 8x16 tilemap  [Assembly Coder's Zenith]
;  input: HL = pointer to tilemap to draw
;  total: 34b/91529t (including ret and entire GridPutSprite)
;============================================================
PutCol:
 ld d,0
 ld e,0
 ld a,0
 ld (Xoffset),a
 ld a,(Tile)
 cp 44
 jp z,Wait
 inc a
 ld (Tile),a
 ld b,a
ColR:
 ld a,0
 cp b
 jp z,ColL
 inc hl 
 inc hl 
 inc hl 
 inc hl 
 inc hl 
 inc hl 
 inc hl 
 inc hl
 dec b 
 jr ColR
 
ColL:
 push hl                ; save map pointer
 ld l,(hl)              ; get the current tile
 ld h,0                 ; clear the upper byte
 add hl,hl              ; multiply by 8 (* 2)
 add hl,hl              ; * 4
 add hl,hl              ; * 8
 ld bc,Sprites          ; load the start of the sprites
 add hl,bc              ; add togehter to get correct sprite
 push de                ; save sprite draw location
 call GridPutSprite     ; draw the sprite
 pop de                 ; restore sprite draw location
 pop hl                 ; restore map pointer
 inc hl                 ; go to next tile
 inc d                  ; move location to the right
 bit 3,d                ; if the 4th bit (e = 8), we're done
 jr z,ColL          ; only jump if the row isn't complete
 ld d,0                 ; draw at the left again
 inc e                  ; move location down a row
 bit 0,e                ; check if it's done (e = 8)
 ret nz                 ; return if we're done
 jr ColL          ; jump back to the top of the loop
 
;===========================================================
; GridPutSprite:                       [routine by Dan Eble]
;  Puts an aligned sprite at (E, D), where HL -> Sprite
;===========================================================
; IN : D  = y (0-7 inclusive)
;      E  = x (0-15 inclusive)
;      HL-> sprite
;
; OUT: AF, BC, DE, HL, IX modified
; Current total : 28b/567t (not counting ret)
;===========================================================

GridPutSprite:
 ld e,15
 push hl
 pop ix					; ix-> sprite
 srl d          		; de = 128y+x (16 bytes/row, 8 rows)
 rra
 and $80
 or e           		; add x offset (remember x <= 15)
 ld e,a
 ld hl,Buffer            ; hl-> video ram
 add hl,de              ; hl-> video ram + offset
 ld b,8                 ; initialize loop counter
 ld de,$10              ; initialize line increment

GPS_Loop
 ld a,(ix+0)    		; get byte from sprite
 ld (hl),a      		; put byte on screen
 inc ix         		; move to next byte in sprite
 add hl,de     			; move to next line on screen
 djnz GPS_Loop
 ret


Map1:
    .db 0,0,0,0,0,0,0,0
    .db 2,2,2,2,2,2,2,2
    .db 0,0,0,0,0,0,0,2
    .db 0,1,1,1,1,1,0,2
    .db 0,1,0,1,0,1,0,2
    .db 0,0,1,0,1,0,0,2
    .db 0,0,0,0,0,0,0,2
    .db 0,0,1,1,1,0,0,2
    .db 0,1,0,1,0,1,0,2
    .db 0,1,0,1,0,1,0,2
    .db 0,0,1,1,0,0,0,2
    .db 0,0,0,0,0,0,0,2
    .db 0,1,1,1,1,1,0,2
    .db 0,0,1,0,0,0,0,2
    .db 0,0,1,0,0,0,0,2
    .db 0,0,0,1,1,1,0,2
    .db 0,0,0,0,0,0,0,2
    .db 0,0,0,0,0,0,0,2
    .db 0,1,1,1,1,0,0,2
    .db 0,0,0,0,0,1,0,2
    .db 0,0,0,0,0,1,0,2
    .db 0,1,1,1,1,0,0,2
    .db 0,0,0,0,0,1,0,2
    .db 0,0,0,0,0,1,0,2
    .db 0,1,1,1,1,0,0,2
    .db 0,0,0,0,0,0,0,2
    .db 0,0,1,1,1,0,0,2
    .db 0,1,0,1,0,1,0,2
    .db 0,1,0,1,0,1,0,2
    .db 0,0,1,1,0,0,0,2
    .db 0,0,0,0,0,0,0,2
    .db 0,1,1,1,1,1,0,2
    .db 0,0,0,1,0,1,0,2
    .db 0,0,0,0,1,0,0,2
    .db 0,0,0,0,0,0,0,2
    .db 0,0,1,1,1,0,0,2
    .db 0,1,0,1,0,1,0,2
    .db 0,1,0,1,0,1,0,2
    .db 0,0,1,1,0,0,0,2
    .db 0,0,0,0,0,0,0,2
    .db 0,1,1,1,1,1,0,2
    .db 0,0,1,0,0,0,0,2
    .db 0,0,1,0,0,0,0,2
    .db 0,0,1,0,0,0,0,2
    .db 0,0,0,0,0,0,0,2
    .db 2,2,2,2,2,2,2,2


Sprites:
sprite_0:
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
sprite_1:
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
sprite_2:
	.db %11111111
	.db %10001000
	.db %10001000
	.db %11111111
	.db %00100010
	.db %00100010
	.db %11111111
	.db %10001001

Tile:
   .db 0
Xoffset:
   .db 0
Buffer: 
   .db 0

TitleSprite:
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$c3,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$1c,$80,$00,$00,$01,$c7,$01,$ff,$80,$00,$00,$00,$00,$00,$00
 .db $00,$33,$80,$00,$00,$00,$c3,$00,$60,$e0,$00,$00,$00,$00,$00,$00
 .db $00,$61,$80,$00,$00,$00,$c3,$00,$60,$70,$00,$00,$00,$00,$00,$00
 .db $00,$60,$80,$00,$00,$00,$c3,$00,$60,$30,$00,$00,$00,$00,$00,$00
 .db $00,$70,$83,$c6,$e1,$c0,$c3,$00,$60,$18,$78,$ce,$38,$1c,$00,$00
 .db $00,$38,$06,$6f,$66,$70,$c3,$00,$60,$18,$cd,$f3,$cc,$67,$00,$00
 .db $00,$1e,$0c,$26,$04,$30,$c3,$00,$60,$19,$86,$c3,$0c,$43,$00,$00
 .db $00,$07,$0c,$06,$0c,$18,$c3,$00,$60,$19,$fe,$c3,$0c,$c1,$80,$00
 .db $00,$03,$8c,$06,$0c,$18,$c3,$00,$60,$19,$80,$c3,$0c,$c1,$80,$00
 .db $00,$41,$cc,$06,$0c,$18,$c3,$00,$60,$19,$80,$c3,$0c,$c1,$80,$00
 .db $00,$40,$cc,$06,$0c,$18,$c3,$00,$60,$31,$80,$c3,$0c,$c1,$80,$00
 .db $00,$60,$ce,$26,$06,$10,$c3,$00,$60,$61,$c2,$c3,$0c,$61,$00,$00
 .db $00,$71,$87,$c6,$06,$30,$c3,$00,$60,$c0,$fc,$c3,$0c,$63,$00,$00
 .db $00,$4f,$03,$8f,$03,$c1,$e7,$81,$ff,$80,$79,$e7,$9e,$3c,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$fc,$00,$00,$0f,$79,$e0,$10,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$22,$00,$00,$02,$10,$80,$30,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$21,$00,$00,$02,$10,$80,$10,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$21,$00,$00,$01,$10,$80,$10,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$22,$0e,$2c,$01,$09,$07,$16,$0e,$2c,$00,$00,$00,$00
 .db $00,$00,$00,$3e,$11,$72,$01,$19,$08,$99,$11,$70,$00,$00,$00,$00
 .db $00,$00,$00,$21,$1f,$22,$00,$a9,$0f,$90,$9f,$20,$00,$00,$00,$00
 .db $00,$00,$00,$20,$90,$22,$00,$a5,$08,$10,$90,$20,$00,$00,$00,$00
 .db $00,$00,$00,$20,$90,$22,$00,$a5,$08,$10,$90,$20,$00,$00,$00,$00
 .db $00,$00,$00,$21,$11,$22,$00,$42,$08,$91,$11,$20,$00,$00,$00,$00
 .db $00,$00,$00,$fe,$0e,$77,$00,$42,$07,$0e,$0e,$70,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 .db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

.end
